pro n2xx 
; main Routine for n2XX
; initializes the data environment
; and the graphic user interface,
; starts the program flow



;define the global variables 

;contains all widgets (and some necessary parameters) of the main program that need to be accessed from other procedures
COMMON widgets, base, draw, load_button, exit_button, targetid_val, startdat_val, enddat_val, alpha_val, delta_val, $
       starname_val, spectype_val, teff_val, metal_val, mv_val, xLab, yLab, xPlot, yPlot, plot_button, oplot_button, $
       third_base, fourth_base, info_text, exclude_saa, exclude_interpol, exclude_incorrect, range_orig_button, view_header, $
       export_header, dft_button, star_sel, star_sel_value  
       
;contains all data from the CoRot-files that need to be accessed from other procedures       
COMMON data, n2dat, clip_inds, bits, star_ind

;parameters for the gui
COMMON gui, xind, yind, xdata, ydata, x_axis, y_axis, colors,colorind, pressed, pixID, drawID, x_mouse, y_mouse, xrang, yrang, $
       plot_inds, plot_ind, plot_star_ind, inv, prev_zoom, dragged

;contains all widgets of the 'export'-subroutine that need to be accessed from other procedures
COMMON export_widgets, col1, col2, col3, col4, col5, col6, col7, col8, export_base, selection

;contains all widgets of the 'dft'-subroutine that need to be accessed from other procedures
COMMON dft_widgets, dft_x, dft_y, dft_base, selected, dft_selection, dft_drawID, llim, ulim, llim_box, ulim_box,spec, calced, $
       prev_zoom_dft, xrang_dft, yrang_dft,unzoom_dft, llim_c, ulim_c, status_dft, os_box,osratio, finish, cancel_calc, cancel, $
       abort_base,xtit,ytit,x_info, y_info

;define the class for CoRot-data :
;  'headers' : will be as string array containing all the headers of the fits extensions, each header can 
;            have a maximum of 1000 lines
;  'tables' : is a two-dimensional pointer array used to point at dynamically created heap variables which will contain 
;           1) A description of every column in the binary tables in the form of  'Extension_Number -> Field_Description' 
;           2) the content of each column in the binary tables of all the extensions (regardless of type)
;  'tables_clip' : an exact copy of 'tables', used to be modified according to user input (clipping, etc.),
;           which allows 'tables' to always include all the original data points
;  'tables_type' : contains integers representing the data type of each column
;
;  'ind'  : an integer array containing the number of the extension for each column of data in the 'tables' data structure
;
struct = {n2obj, headers : ptr_new(/allocate_heap), tables : ptr_new(/allocate_heap), tables_clip : ptr_new(/allocate_heap), $
          tables_type : ptr_new(/allocate_heap), strings : ptr_new(/allocate_heap), export_strings : ptr_new(/allocate_heap), $ 
	  ind : ptr_new(/allocate_heap)}

;settings for the graphic device

if (!d.name eq 'X') then device, TRUE = 24
device, get_screen_size=ss
device, retain=2
device, decomposed=0
loadct,40, /silent

;define a number of colors for successive overplotting
colors = reverse((indgen(10)+1)*25)
;set the color index for the current plot

;initialize some global parameters
dragged = 0
star_ind = -1
useClip = -1
xind = -1
yind = -1
xdata = 0.D
ydata = 0.D
x_axis = [0., 0.]
y_axis = [0., 0.]
prev_zoom = [0., 0., 0., 0.]
bits = 0L
pixID = -1
pressed = 0
 
x_mouse = [0.,0]
y_mouse = [0.,0]
xrang = [-1.0, -1.0]
yrang = [0., 0.]
plot_inds = intarr(500,2)
clip_inds = intarr(500)
plot_inds[*,*] = -1
plot_ind = 0
plot_star_ind = 0

!P.position=[0.1,0.12,0.95,0.95]
inv = 0
star_sel_value = ['No data loaded']

;create the widgets for the main program
base = WIDGET_BASE(title='CoRoT n2 eXplorer/eXporter - (c) Michael Gruberbauer, 2008', xsize=1024, ysize=810)
control_base = WIDGET_BASE(base, frame=1, xsize=1024, ysize=180)
data_base = WIDGET_BASE(base, frame=1, xsize=1024, ysize=620,column=2,yoffset=180)

first_base = WIDGET_BASE(control_base, xsize=224, row=5, frame=1)
load_button = WIDGET_BUTTON(first_base, value='Load n2 data', uvalue='load', xsize=224, ysize=60)
about_button = WIDGET_BUTTON(first_base,value='About n2XX', uvalue='about',xsize=224)
exit_button = WIDGET_BUTTON(first_base, value='     EXIT     ', uvalue='exit', xsize=224)
star_sel = WIDGET_DROPLIST(first_base,value=star_sel_value, xsize=224, uvalue='star_sel')

second_base = WIDGET_BASE(control_base, xoffset=235, xsize=224, row=7, frame=1)
xLab = WIDGET_LABEL(second_base, value='Select X : (extension -> field) ', xsize=224, uvalue='selX')
xPlot = WIDGET_DROPLIST(second_base, value='No data', uvalue='xdata')
yLab = WIDGET_LABEL(second_base, value='Select Y : (extension -> field) ', xsize=224, uvalue='selY')
yPlot = WIDGET_DROPLIST(second_base, value='No data', uvalue='ydata')
plot_base = WIDGET_BASE(second_base, column=2)
plot_button = WIDGET_BUTTON(plot_base, value='Plot', sensitive=0, xsize=110, uvalue='plot')
oplot_button = wIDGET_BUTTON(plot_base,value='Overplot', sensitive=0, xsize=110, uvalue='oplot')
range_orig_button = WIDGET_BUTTON(second_base, value='Total view', sensitive=0, xsize=224, uvalue='orig')

third_base = WIDGET_BASE(control_base, xoffset=470, xsize=140, row=5, frame=1, sensitive=0)
exclude = WIDGET_LABEL(third_base, value='Exclude data points')
exclude2 = WIDGET_LABEL(third_base, value='with the following')
exclude3 = WIDGET_LABEL(third_base, value='flag :')
exclude4 = WIDGET_LABEL(third_base, value='------------------')
exclude_base = WIDGET_BASE(third_base, /nonexclusive)
exclude_saa = WIDGET_BUTTON(exclude_base, value='crossing SAA',uvalue='crossing')
exclude_interpol = WIDGET_BUTTON(exclude_base, value='interpolated', uvalue='interpol')
exclude_incorrect = WIDGET_BUTTON(exclude_base, value='incorrect value', uvalue='incorrect')

fourth_base = WIDGET_BASE(control_base, xoffset=620, xsize=160, column=1, frame=1,sensitive=0)
fifth_base = WIDGET_BASE(control_base, xoffset=790, xsize=220, column=1, frame=1,sensitive=1)
export_button = WIDGET_BUTTON(fourth_base,value='>EXPORT Time Series<',uvalue='export', xsize=180)
dft_button = WIDGET_BUTTON(fourth_base, value='>CALCULATE DFT<', uvalue='calc_dft', xsize=180, sensitive=0)

info_text = WIDGET_TEXT(fifth_base, value=['Welcome to n2XX ! (V1.2a)','-------------------------------'], ysize=10, /scroll)

info_base = WIDGET_BASE(data_base, xsize=200, ysize=640,row=22, frame=1)
draw = WIDGET_DRAW(data_base, xsize=800, ysize=620, scr_xsize=800, scr_ysize=620, uvalue='draw', /BUTTON_EVENTS, /MOTION_EVENTS)

starname = WIDGET_LABEL(info_base, value='Name of star :')
starname_val = WIDGET_TEXT(info_base, value='No data', xsize=30)
targetid = WIDGET_LABEL(info_base,value='ID of the CoRoT target :')
targetid_val = WIDGET_TEXT(info_base,value='No data', xsize=30)
startdat = WIDGET_LABEL(info_base,value='UT date of first measurement :')
startdat_val = WIDGET_TEXT(info_base,value='No data', xsize=30)
enddat = WIDGET_LABEL(info_base,value='UT date of last measurement :')
enddat_val = WIDGET_TEXT(info_base,value='No data', xsize=30)
alpha = WIDGET_LABEL(info_base,value='RA :')
alpha_val = WIDGET_TEXT(info_base,value='No data', xsize=30)
delta = WIDGET_LABEL(info_base,value='Dec :')
delta_val = WIDGET_TEXT(info_base,value='No data', xsize=30)
spectype = WIDGET_LABEL(info_base,value='Spectral type :')
spectype_val = WIDGET_TEXT(info_base, value='No data', xsize=30)
mv = WIDGET_LABEL(info_base,value='Mv (abs. mag.) :')
mv_val = WIDGET_TEXT(info_base, value='No data', xsize=30)
teff = WIDGET_LABEL(info_base,value='T_eff :')
teff_val = WIDGET_TEXT(info_base, value='No data', xsize=30)
metal = WIDGET_LABEL(info_base,value='Metallicity :')
metal_val = WIDGET_TEXT(info_base, value='No data', xsize=30)

view_header = WIDGET_BUTTON(info_base, value='Show Header(s)', xsize=200, uvalue='view_header', sensitive=0)
export_header = WIDGET_BUTTON(info_base, value='Export Header(s)', xsize=200, uvalue='exhead', sensitive=0)

;initialize a pointer for a future CoRot-Data-object 
n2dat = obj_new()

;realize the widget
WIDGET_CONTROL,base,/realize
WIDGET_CONTROL,draw,get_value=drawID

;start the program flow ...
XMANAGER, 'n2extr', base, /no_block, cleanup='cleanup_n2xx'
end

;-----------------------------------------------------------------------------

function check_type, i

COMMON data

if i lt 0 then return, 0 else begin
   if n2dat[star_ind]->getType(i) eq 7 then begin
      inform = dialog_message(['Warning : This column does not contain numerical values!' , 'Please select a different column.'], $
                           /information, title='Data type problem encountered ...')
      if strcmp(inform,'OK') then return, 0   
   endif else begin
      return, 1
   endelse
endelse

end

;-----------------------------------------------------------------------------

pro load_data, filesel
; handles the loading of CoRot-data

;acces to specific global variables
COMMON data
COMMON widgets
COMMON gui

tomag = -1
nfiles = n_elements(filesel)


;initialize some little helpers ...
for f=0.,nfiles-1 do begin
   unit = 0
   error = 0
   ext = 1
   find = 0
   h =''
 
  ;catch any error for the file I/O
   catch, error
   WIDGET_CONTROL, base, /hourglass 
   if error eq 0 then begin
      ;prepare the GUI for the data
      send_message,'Loading ' + strtrim(string(f+1,FORMAT='(I)'),2) + ' of ' + strtrim(string(nfiles,FORMAT='(I)'),2) + ' ...'   
      bits = 0L
      WIDGET_CONTROL, exclude_saa, set_button=0
      WIDGET_CONTROL, exclude_interpol, set_button=0
      WIDGET_CONTROL, exclude_incorrect, set_button=0
      ;open fits and get number of extensions
      fxbopen,unit,filesel[f], 1
      fxfindend, unit, ext
      fxbclose, unit
      
           ;read fits to extract headers of extensions and to 
      ;find total number of columns in all binary tables
      n_columns = 0
      data_struc = ptrarr(ext,/allocate_heap)
      headers = strarr(ext,1000)
      
      
      
      ;check if the user wants to convert flux to magnitudes and set tomag accordingly
      if tomag eq -1 then begin
         ask = dialog_message('Do you want to convert flux to relative flux ?',/DEFAULT_NO, /QUESTION, title='Convert ?')
         if strcmp(ask, 'Yes') then tomag=1 else tomag=0
      endif      
      
      for i=0., ext-1 do begin
         temp = mrdfits(filesel[f],i,h)  
	 if i ne 0 then begin
            data_struc[i] = ptr_new(temp)
	    n_columns += n_tags(temp)
	 endif   
         n_strs = size(h)
         headers[i,0:n_strs[1]-1] = h	    
      endfor
      
      ;initialize the dynamical data structures
      tables = ptrarr(n_columns,2,/allocate_heap)
      tables_clip = ptrarr(n_columns,2,/allocate_heap) 
      tables_type = ptrarr(n_columns, /allocate_heap) 
      extind = intarr(n_columns)
      index = 0
            
      ;go through all extensions and extract all information
      for i=1., ext -1 do begin
         n_col = n_tags(*data_struc[i])
         for j=0.,n_col-1 do begin
	       ;n_columns = n_columns + FXPAR(h,'TFIELDS')
	       temp = (*data_struc[i]).(j)
	       field_descr = FXPAR(reform(headers[i,*]),'TTYPE'+strtrim(string(j+1,FORMAT='(I)'),2))
   	       unit_descr = FXPAR(reform(headers[i,*]),'TUNIT'+strtrim(string(j+1,FORMAT='(I)'),2))
	       if ((strpos(unit_descr, 'electron') ne -1) || (strpos(unit_descr, 'ELECTRON') ne -1)) && (tomag eq 1) then temp /= mean(temp,/double)
	       *tables[index,0] =  strtrim(string(i,FORMAT='(I)'),2) + ' -> ' + strjoin(field_descr,/single)
  	       *tables[index,1] = temp
   	       *tables_clip[index,0] =  strtrim(string(i,FORMAT='(I)'),2) + ' -> ' + strjoin(field_descr,/single)
   	       *tables_clip[index,1] = temp
	       *tables_type[index] = size(temp,/type)
	       extind[index] = i
	       index = index + 1 
	 endfor   
      endfor    
               
        
      ; Create the droplist-entries for the plotting option 
      strings = 'Select Data'
      for i=0.,index-1 do begin
   	  strings = [strings[*],*tables[i,0]]
      endfor
      
      ; set the variable for the 'Export' option 
      export_strings = strings
      export_strings[0] = 'No Data'
   	 
      ;create the n2dat object that contains all the data
      if n2dat[0] eq obj_new() then begin
   	 n2dat = obj_new('n2obj', headers, tables, tables_clip, tables_type, strings, export_strings, extind)
      endif else begin
   	 n2dat = [n2dat[*],obj_new('n2obj', headers, tables, tables_clip, tables_type, strings, export_strings, extind)]
      endelse
      close,1
       
      ;increase the index for the number of loaded individual n2-data-files
      star_ind = star_ind + 1
      id = FXPAR(reform(headers[0,*]),'COROTID', count=find)
      if find eq 0 then id = 'Data set ' + strtrim(string(star_ind,FORMAT='(I)'),2) else id = 'CoRoT ID ' + strtrim(string(FXPAR(reform(headers[0,*]),'COROTID'),FORMAT='(I)'),2)
   
      if strcmp(star_sel_value[0],'No data loaded') eq 1 then star_sel_value = id  $
      else star_sel_value = [star_sel_value[*],id]
      WIDGET_CONTROL,star_sel,set_value=star_sel_value
      WIDGET_CONTROL,star_sel,set_droplist_select=star_ind
      
      ; enable full functionality of the program, now that data has been loaded 
      display_starinf,star_ind
      WIDGET_CONTROL,plot_button,sensitive=0   
      WIDGET_CONTROL,third_base,sensitive=1
      WIDGET_CONTROL,fourth_base,sensitive=1
      WIDGET_CONTROL,view_header, sensitive=1
      WIDGET_CONTROL,export_header, sensitive=1
      WIDGET_CONTROL,dft_button, sensitive=1
      
      
      send_message, 'Loading done !'
       send_message,'-------------------'

   ;reading is done - if an error occured, send a message  
   endif else send_message,'Error - check file !'
endfor

WIDGET_CONTROL,base,hourglass=0
;ptr_free,tables, tables_type, tables_clip
ptr_free, data_struc

end
 
;-----------------------------------------------------------------------------

pro display_starinf, i

COMMON widgets
COMMON data

headers = n2dat[i]->getHeaders()
 
 
;set values for the information display in the GUI taken from the primary header
   ;------------------------------------------------------------
   startdat = FXPAR(headers,'STARTDAT', count=find)
   if find eq 0 then startdat = 'No data'
   WIDGET_CONTROL,startdat_val,set_value=startdat
   
   enddat = FXPAR(headers,'END_DATE', count=find)
   if find eq 0 then enddat = 'No data'  
   WIDGET_CONTROL,enddat_val,set_value=enddat
   
   id = FXPAR(headers,'COROTID', count=find)
   if find eq 0 then id = 'No data' else begin
      id = strtrim(string(FXPAR(headers,'COROTID'),FORMAT='(I)'),2)
   endelse
   WIDGET_CONTROL,targetid_val,set_value=id
   
   alpha = FXPAR(headers,'ALPHA', count=find)
   if find eq 0 then begin
      alpha = 'No data'
      WIDGET_CONTROL,alpha_val,set_value=alpha
   endif else begin
      alpha_hours = strtrim(string(((alpha/15.0 - ((alpha/15.0) mod 1))),FORMAT='(I)'),2)
      alpha_minutes = strtrim(string((((alpha/15.0) mod 1)*60.0D) - (((alpha/15) mod 1)*60.0D mod 1),FORMAT='(I)'),2)
      alpha_seconds = strtrim(string((((alpha/15.0) mod 1)*60.0D - alpha_minutes)*60.0D,FORMAT='(F7.2)'),2)
      WIDGET_CONTROL,alpha_val,set_value=alpha_hours + ' h ' + alpha_minutes + ' min ' + alpha_seconds + ' sec' 
   endelse
   
   delta = FXPAR(headers,'DELTA', count=find)
   if find eq 0 then begin
      delta='No data'
      WIDGET_CONTROL,delta_val,set_value=delta
   endif else begin
      delta_deg = strtrim(string((delta - (delta mod 1)),FORMAT='(I)'),2)
      if delta lt 0 then delta = delta*(-1.0D)
      delta_minutes = strtrim(string((delta mod 1)*60.0D - ((delta mod 1)*60.0D mod 1), FORMAT='(I)'),2)
      delta_seconds = strtrim(string(((delta mod 1)*60.0D mod 1)*60.0D, FORMAT='(F7.2)'),2)
      WIDGET_CONTROL,delta_val,set_value=delta_deg + ' d ' + delta_minutes + ''' ' + delta_seconds + ''''''
   endelse
   
   starname = FXPAR(headers,'STARNAME', count=find)
   if find eq 0 then starname = 'No data'
   WIDGET_CONTROL,starname_val, set_value=starname
   
   spectype = FXPAR(headers,'SPECTYPE', count=find)
   if find eq 0 then spectype = 'No data' else begin
      subclass = FXPAR(headers,'SUBCLASS', count=find)
      if find ne 0 then spectype = spectype + subclass
      lumclass = FXPAR(headers,'LUMCLASS', count=find)   
      if find ne 0 then spectype = spectype + lumclass
   endelse 
   WIDGET_CONTROL,spectype_val, set_value=strjoin(strsplit(spectype,' ',/extract),/single)
   
   mv = FXPAR(headers,'ABSM_V', count=find)
   if find eq 0 then mv = 'No data' else mv = strtrim(string(mv, FORMAT='(F7.3)'),2)
   WIDGET_CONTROL, mv_val, set_value=mv
   
   teff = FXPAR(headers,'TEFF', count=find)
   if find eq 0 then teff = 'No data' else teff = strtrim(string(teff, FORMAT='(I)'),2)
   WIDGET_CONTROL, teff_val, set_value=teff

   metal = FXPAR(headers,'METAL', count=find)
   if find eq 0 then metal = 'No data' else metal = strtrim(string(metal, FORMAT='(F7.3)'),2)
   WIDGET_CONTROL, metal_val, set_value=metal 
   
   strings = n2dat[i]->getStrings()
   
   WIDGET_CONTROL,xPlot,set_value=strings
   WIDGET_CONTROL,yPlot,set_value=strings 
end

;-----------------------------------------------------------------------------
pro update_graphix

COMMON data
COMMON gui
COMMON widgets

if plot_inds[0,0] ne -1 then begin
   if mean(xrang) eq -1 then begin
      inv = 1
      for i=0.,plot_ind do begin
          ;see if clipped data is desired and get data
          if clip_inds[i] eq 0 then begin
   	     xdata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,0])
   	     ydata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,1])
	  endif else begin
	     xdata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,0], /clip)
   	     ydata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,1], /clip)
	  endelse
   	 ;determine the axis range
   	  xrang_temp = (max(xdata)-min(xdata))/250.0D*[-1D,1D] + [min(xdata), max(xdata)]
	  if i eq 0 then xrang = xrang_temp $
	  else begin
   	     if xrang_temp[0] lt xrang[0] then xrang[0] = xrang_temp[0]
   	     if xrang_temp[1] gt xrang[1] then xrang[1] = xrang_temp[1]
	  endelse
   	  yrang_temp = (max(ydata)-min(ydata))/50.0D*[-1D,1D] + [min(ydata), max(ydata)]   
   	  if i eq 0 then yrang = yrang_temp $
	  else begin	  
  	     if yrang_temp[0] lt yrang[0] then yrang[0] = yrang_temp[0]
   	     if yrang_temp[1] gt yrang[1] then yrang[1] = yrang_temp[1]
	  endelse
	  if mean(ydata) gt 0 then inv = 0 
      endfor
      
      prev_zoom = [xrang[*], yrang[*]]
   endif
   ;if inv eq 1 then yrang = yrang[reverse(sort(yrang))]
 
   loadct,0, /silent
   ;produce the coordinate system
   plot, xrang,yrang, ystyle=1, xstyle=1, xrange=xrang, yrange=yrang, background=255, color=0, /nodata      

   for i=0., plot_ind do begin 
      ;load the appropriate color table
      if i gt 0 then loadct,40, /silent else loadct,0, /silent  
      ;see if clipped data is desired and get data   
      if clip_inds[i] eq 0 then begin
         xdata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,0])
         ydata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,1])
      endif else begin
         xdata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,0], /clip)
         ydata = n2dat[plot_star_ind[i]]->getData(plot_inds[i,1], /clip)
      endelse      
      ;produce the plot
      if i gt n_elements(colors)-1 then col = colors[(i mod n_elements(colors)-1)] else col = colors[i]
      if i eq 0 then oplot, xdata, ydata, psym=1, symsize=0.2, color=0 else oplot, xdata, ydata, psym=1, symsize=0.2,color=col
   endfor   
endif

end
;-----------------------------------------------------------------------------

pro n2extr_event, ev
; event handler for the main program

COMMON widgets
COMMON data
COMMON gui

;set the flag for loading new data to zero and get the uvalue of the widget that
;caused the event
flag = 0
WIDGET_CONTROL, ev.id, get_uvalue=uval

case uval of 'load' : begin                         
                         ;manage the loading of data
                         if n2dat[0] ne obj_new() then begin
			    ask = dialog_message('Load additional data ? ',/DEFAULT_NO, /QUESTION, title='Load new data ?')
	                    if strcmp(ask,'Yes') then flag = 1 else flag = 0
		         endif else flag = 1
			 if flag eq 1 then begin
			    filesel = DIALOG_PICKFILE(/read,/must_exist,/multiple)
			    if filesel[0] then begin 
			       load_data, filesel			       
			    endif   
		         endif               
	              end
		      
	       'star_sel' : begin
	                 display_starinf,ev.index
			 star_ind = ev.index
			end
			    
			
	       'exit' : begin
	                   WIDGET_CONTROL, base, /destroy  ;manage exiting
			   cleanup_n2xx
			end
	       
	       'calc_dft' : create_dft 
	       
	       'view_header' : create_headerview 
	       
	       'xdata' : begin
	                    ;change the data for the x-coordinate
	                    if check_type(ev.index-1) then begin
			       xind = ev.index-1
			       if (xind ne -1) and (yind ne -1) then WIDGET_CONTROL,plot_button,sensitive=1		      		  		  
			    endif else begin
			       WIDGET_CONTROL,ev.id,set_droplist_select=0	
			       WIDGET_CONTROL,plot_button,sensitive=0
			       WIDGET_CONTROL,oplot_button,sensitive=0
			       xind = -1		      
		            endelse  			    
	                 end
			 
	       'ydata' : begin
	                    ;change the data for the x-coordinate
			    if check_type(ev.index-1) then begin
			       yind = ev.index-1
			       if (xind ne -1) and (yind ne -1) then WIDGET_CONTROL,plot_button,sensitive=1		      		  		  
			    endif else begin
			       WIDGET_CONTROL,ev.id,set_droplist_select=0	
			       WIDGET_CONTROL,plot_button,sensitive=0	
			       WIDGET_CONTROL,oplot_button,sensitive=0	      
			       yind = -1
		            endelse   
	                 end
	       
	       'plot' :  begin
	                    ;manage plotting
			    send_message,'Plotting - //////////////'
			    send_message,'click and drag to zoom  /'
			    send_message,'CTRL and click to unzoom/'
			    send_message,'/////////////////////////'
			    xrang = [-1, -1]	                    
			    plot_inds[*,*] = -1
			    plot_star_ind = star_ind
			    plot_ind = 0
			    plot_inds[0,0] = xind
			    plot_inds[0,1] = yind
			    if bits eq 0 then clip_inds[0] = 0 $
			    else clip_inds[0] = 1
	                    update_graphix
			    x_axis = [!P.position[0],!P.position[2]]*800
			    y_axis = [!P.position[1],!P.position[3]]*620
			   		
			    ;enable overplotting
			    WIDGET_CONTROL,oplot_button,sensitive=1			   
			 end
	       'oplot' : begin
	                    xrang = [-1, -1]
	                    ;manage overplotting 
	                    plot_ind = plot_ind + 1 
			    plot_star_ind = [plot_star_ind[*],star_ind]
			    plot_inds[plot_ind,0] = xind
			    plot_inds[plot_ind,1] = yind   
			    if bits eq 0 then clip_inds[plot_ind] = 0 $
			    else clip_inds[plot_ind] = 1
			    update_graphix			    
			 end
	       'orig'  : begin 
	                    xrang = [-1,-1]
			    update_graphix
			 end 	 	 
			 
	       'draw'  : begin
	              ;handle the zooming
		      case ev.modifiers of 0 : begin
		      ;no modifier key is pressed
                            case ev.type of 			      
	                       0 : begin
			              ;mouse button has been pressed - set flag, get coordinates
			              pressed = 1
	                              x_mouse[0] = ev.x
		                      y_mouse[0] = ev.y				 
				      ;copy plot to pixel map     
				      wset,drawID
			              window,/free,/pixmap,xsize=800, ysize=620
			              pixID = !D.Window			              
				      device, copy=[0, 0, 800, 620, 0, 0, drawID]				      
			           end
		               1 : begin
			              ;mouse button has been released - unset flag, get coordinates
			              if dragged eq 1 then begin				      
				         pressed = 0					 
		                         x_mouse[1] = ev.x
		                         y_mouse[1] = ev.y						   
				         ;clear events, copy pixel map to plotting window				
		  	                 WIDGET_CONTROL, ev.id, clear_events=1
			                 wset,drawID
			                 device, copy=[0, 0, 800, 620, 0, 0, pixID]
			                 wdelete, pixID   
				         ;convert pixel coordinates to data coordinates  
			                 temp = convert_coord(x_mouse[sort(x_mouse)], y_mouse[sort(y_mouse)], /DEVICE, /TO_DATA)
				         ;set the previous axis range for un-zooming
				         prev_zoom = [xrang[*], yrang[*]]
				         ;set the new axis range
				         xrang = reform(temp[0,*])
				         yrang = reform(temp[1,*])				 
				         x_mouse = [0,0]
			                 y_mouse = [0,0]
				         ;enable the 'Total view' button				 
			                 WIDGET_CONTROL, range_orig_button, sensitive=1
				         ;update the graph
			                 update_graphix 
					 dragged = 0
				      endif else pressed = 0	 				 
			           end
		               2 : begin	
			              ;mouse has been moved -> check if zoom flag is set 
			              if pressed eq 1 then begin
				         dragged = 1
				         ;copy the previous plot to the drawing window and include a rectangle showing the zoom area
  				         wset, drawID
			                 loadct, 40, /silent
			                 device, copy=[0, 0, 800, 620, 0, 0, pixID]
			                 plots, [x_mouse[0], x_mouse[0], ev.x, ev.x, x_mouse[0]], $
					        [y_mouse[0], ev.y, ev.y, y_mouse[0], y_mouse[0], y_mouse[0]], /device, color=255
		                      endif				     
				   end
			    endcase
			 end   
			;CTRL is pressed 
			 2 : if ev.type eq 1 then begin
			        ;modifier key 'CTRL' + mouse click -> set axis to the previous axis range				
			        xrang = prev_zoom[0:1]
				yrang = prev_zoom[2:3]
				;update graph
				update_graphix
			     endif
			 
			 else : break    	   
		      endcase
	       end
	       
	       'interpol' : begin
	                       ;set (ev.select = 1) or unset (ev.select = 0) the appropriate bit for interpolated values    
	                       if ev.select eq 1 then begin
			         bits = bits + 8
			       endif else begin
			         bits = bits - 8
			       endelse	 
			       ;clip the data 
			       n2dat[star_ind]->clipData,bits    		
	                    end
	       'incorrect' : begin
	                       ;set (ev.select = 1) or unset (ev.select = 0) the appropriate bit for incorrect values    
	                       if ev.select eq 1 then begin
			         bits = bits + 3072
			       endif else begin
			         bits = bits - 3072
			       endelse
			       ;clip the data 	       
			       n2dat[star_ind]->clipData,bits 
			            	
	                     end
	       
	       'crossing' : begin
	                       ;set (ev.select = 1) or unset (ev.select = 0) the appropriate bit for values obtained while above the SAA  
	                       if ev.select eq 1 then begin
			         bits = bits + 4
			       endif else begin
			         bits = bits - 4
			       endelse
			       ;clip the data
			       n2dat[star_ind]->clipData,bits 
			       
	                    end
	       'export' :  create_export ;create the 'data export' widget
	       
	       'about' : create_credits  ;create the 'about' widget
	       
	       'exhead' : begin
	                     ;manage exporting the header
                             filesel = DIALOG_PICKFILE(/write)		
		             if filesel then begin
			        send_message,'Writing header ...'
				h = ''
			        WIDGET_CONTROL,base,/hourglass
			        openw,1,filesel
				h = n2dat[star_ind]->getHeaders()
				printf,1,h
				close,1
			     endif
			     send_message,'Done !'
			     send_message,'-------------------'
			  end
				
				
			 			
endcase

end

;-----------------------------------------------------------------------------

function n2obj::init, header, table, table_clip, table_type, string_descr, exp_descr, index
;constructor for CoRot-data objects

self.headers = ptr_new(header)
self.tables = ptr_new(table)
self.tables_clip = ptr_new(table_clip)
self.ind = ptr_new(index)
self.tables_type = ptr_new(table_type)
self.strings = ptr_new(string_descr)
self.export_strings = ptr_new(exp_descr)

return,1 
end

;-----------------------------------------------------------------------------

function n2obj::getData, i, clip=clip
;function to return binary table data with the column index i

if keyword_set(clip) then return, *(*self.tables_clip)[i,1] $
else  return, *(*self.tables)[i,1]

end

;-----------------------------------------------------------------------------

function n2obj::getType, i 
;function to return binary table data with the column index i

return, *(*self.tables_type)[i]

end

;-----------------------------------------------------------------------------

function n2obj::getStrings  
;function to return binary table data with the column index i

return, *self.strings

end

;-----------------------------------------------------------------------------

function n2obj::getExportStrings  
;function to return binary table data with the column index i

return, *self.export_strings

end

;-----------------------------------------------------------------------------

function n2obj::getHeaders
;function to return the header string array

temp_new = ''
temp = *self.headers
dim = size(temp)
for i=0.,dim[1]-1 do begin
  use = where(strcmp(reform(temp[i,*]),'') eq 0)
  if i eq 0 then temp_new = reform(temp[i,use]) else temp_new = [temp_new[*], reform(temp[i,use])]
  temp_new = [temp_new[*],' --------------------------------------------------']
endfor
return, temp_new

end

;-----------------------------------------------------------------------------

pro n2obj::clipData, bits
;clip data according to the user's selection
;bit is a bitmask corresponding to the CoRot 'Status' data format

use = -1 
if bits ne 0 then begin
   for j=1, (*self.ind)[n_elements(*self.ind)-1] do begin 
      find = where(*self.ind eq j)
      for k=0,n_elements(find)-1 do begin
         if strpos(*(*self.tables)[find[k],0], 'STATUS') ne -1 then begin
            status = *(*self.tables)[find[k],1]
	    
            use = where((status AND bits) eq 0)
	    drop = n_elements(status)-n_elements(use)
	    send_message,'Extension ' + strtrim(string(j,FORMAT='(I)'),2) + ' : '
	    send_message,'excluding ' + strtrim(string(drop,FORMAT='(I)'),2) + ' datapoints'
	    send_message,'in total (' + strtrim(string(drop*100D/n_elements(status),FORMAT='(I)'),2) + ' %)'
         endif
      endfor
      if use[0] ne -1 then begin
         for k=0,n_elements(find)-1 do begin
            temp = *(*self.tables)[find[k],1]
            *(*self.tables_clip)[find[k],1] = temp[use]
         endfor
      endif
   endfor
endif
send_message,'-------------------'

end   

;-----------------------------------------------------------------------------

pro n2obj::cleanup
;Destructor for n2obj-objects

ptr_free,self.headers, self.tables, self.tables_clip, self.tables_type, self.strings, self.export_strings, self.ind

end

;-----------------------------------------------------------------------------

pro cleanup_n2xx, widg_id

for i=0.,n_elements(n2dat)-1 do obj_destroy,n2dat[i]

end

;-----------------------------------------------------------------------------

pro create_export
;creates the export widget

COMMON data
COMMON export_widgets
COMMON widgets
COMMON gui

selection = intarr(8)	
selection[*] = -1		 
export_strings = n2Dat[star_ind]->getExportStrings()
export_base = WIDGET_BASE(group_leader=base, title='Export Data ...', /floating, row=2, /modal)
column_base1 = WIDGET_BASE(export_base, column=2)
column_base2 = WIDGET_BASE(export_base, column=2)


first_sub_base = WIDGET_BASE(column_base1, row=4)
column1 = WIDGET_LABEL(first_sub_base, value='Column 1')
col1 = WIDGET_DROPLIST(first_sub_base, value=export_strings, uvalue='col1')
column2 = WIDGET_LABEL(first_sub_base, value='Column 2')
col2 = WIDGET_DROPLIST(first_sub_base, value=export_strings, uvalue='col2', sensitive=0)

second_sub_base = WIDGET_BASE(column_base1, row=4)
column3 = WIDGET_LABEL(second_sub_base, value='Column 3')
col3 = WIDGET_DROPLIST(second_sub_base, value=export_strings, uvalue='col3', sensitive=0)
column4 = WIDGET_LABEL(second_sub_base, value='Column 4')
col4 = WIDGET_DROPLIST(second_sub_base, value=export_strings, uvalue='col4', sensitive=0)

third_sub_base = WIDGET_BASE(column_base2, row=4)
column5 = WIDGET_LABEL(third_sub_base, value='Column 5')
col5 = WIDGET_DROPLIST(third_sub_base, value=export_strings, uvalue='col5', sensitive=0)
column6 = WIDGET_LABEL(third_sub_base, value='Column 6')
col6 = WIDGET_DROPLIST(third_sub_base, value=export_strings, uvalue='col6', sensitive=0)

fourth_sub_base = WIDGET_BASE(column_base2, row=5)
column7 = WIDGET_LABEL(fourth_sub_base, value='Column 7')
col7 = WIDGET_DROPLIST(fourth_sub_base, value=export_strings, uvalue='col7', sensitive=0)
column8 = WIDGET_LABEL(fourth_sub_base, value='Column 8')
col8 = WIDGET_DROPLIST(fourth_sub_base, value=export_strings, uvalue='col8', sensitive=0)

finish = WIDGET_BUTTON(export_base,value='Export...', uvalue='exp')
cancel = WIDGET_BUTTON(export_base,value='Close', uvalue='cancel')

WIDGET_CONTROL,export_base, /realize
XMANAGER, 'export', export_base, /no_block


end

;-----------------------------------------------------------------------------

pro export_event, ev
;event handler for the export widget

COMMON export_widgets
COMMON widgets
COMMON data


WIDGET_CONTROL,ev.id,get_uvalue=uval

case uval of 'exp' : begin
                        ;begin exporting 
    	    	    	filesel = DIALOG_PICKFILE(/write)		
			if filesel then begin
			   ;prepare the gui for exporting
			   send_message,'Writing to file ...'
			   WIDGET_CONTROL,export_base,/hourglass
			   ;open the file selected by DIALOG_PICKFILE
			   openw,1,filesel
			   ;get the data columns that have been selected in the export widget
			   use = where(selection gt -1)
			   s = ''
			   ncols = n_elements(use)
			   selection = selection[use]
			   nlines = lonarr(ncols)
			   colData = ptrarr(ncols,/allocate_heap)
			   formats = strarr(ncols)
			   ;for all columns that have to be exported, obtain the data and check its 
			   ;type to produce correct output
			   for i=0., ncols-1 do begin
			       if bits eq 0 then temp = n2dat[star_ind]->getData(selection[i]) $
			       else temp = n2dat[star_ind]->getData(selection[i], /clip)
			       
			       colData[i] = ptr_new(temp)
			       nlines[i] = n_elements(temp)
			       case size(temp[0],/type) of 1 : formats[i] = '(I)'
			                                   2 : formats[i] = '(I)'
							   3 : formats[i] = '(I)'
							   4 : formats[i] = '(F)'
			                                   5 : formats[i] = '(D)'			                                   
							   7 : formats[i] = '(A)'
			       endcase							   
			   endfor
			   
			   
			   
			   ;write the data to the file
			   
			   ;the maximum number of entries found among the various columns is the number of lines to write,
			   ;if a data column has fewer entries than the total number of lines to write, the remaining lines 
			   ;are filled with zeros
			   for i=0., max(nlines)-1 do begin
			       for j=0.,ncols-1 do begin
			          if i gt nlines[j]-1 then s = s + '   0' else s = s + '   ' + strtrim(string((*colData[j])[i], FORMAT=formats[j]),2)
			       endfor			       
			       printf,1,s
			       s = ''
			   endfor
			   close,1
			   send_message,'Done writing !'
			   send_message,'-------------------'
			   WIDGET_CONTROL, export_base, hourglass=0
			   WIDGET_CONTROL, export_base,/destroy 	   
		       endif		       		       
                     end
	     'cancel' : begin
	                  ;destroy the widget if user wants to cancel
	                  WIDGET_CONTROL, export_base,/destroy
		        end
	     ;handle the selection of columns to export and enable or disable columns according to the number of 
	     ;columns already selected  (i.e., if column 1 has been selected, enable column 2 ...)		
	     'col1' : begin
	                  selection[0] = ev.index-1
			  if ev.index ne 0 then WIDGET_CONTROL, col2, sensitive=1 else WIDGET_CONTROL, col2, sensitive=0
	              end
 	     'col2' : begin
	     	    	  selection[1] = ev.index-1
			  if ev.index ne 0 then WIDGET_CONTROL, col3, sensitive=1 else WIDGET_CONTROL, col3, sensitive=0
	              end
 	     'col3' : begin
	     	    	  selection[2] = ev.index-1 
			  if ev.index ne 0 then WIDGET_CONTROL, col4, sensitive=1 else WIDGET_CONTROL, col4, sensitive=0 
	              end
 	     'col4' : begin
	     	    	  selection[3] = ev.index-1   
			  if ev.index ne 0 then WIDGET_CONTROL, col5, sensitive=1 else WIDGET_CONTROL, col5, sensitive=0
	              end
 	     'col5' : begin
	     	    	  selection[4] = ev.index-1
			  if ev.index ne 0 then WIDGET_CONTROL, col6, sensitive=1 else WIDGET_CONTROL, col6, sensitive=0
	              end
 	     'col6' : begin
	                  selection[5] = ev.index-1
			  if ev.index ne 0 then WIDGET_CONTROL, col7, sensitive=1 else WIDGET_CONTROL, col7, sensitive=0
	              end
 	     'col7' : begin
	                  selection[6] = ev.index-1
			  if ev.index ne 0 then WIDGET_CONTROL, col8, sensitive=1 else WIDGET_CONTROL, col8, sensitive=0
	              end
 	     'col8' : begin
      	    	          selection[7] = ev.index-1 
		      end	      
endcase

end

;-----------------------------------------------------------------------------

pro create_credits
;produce a small widget that contains some info about this program

COMMON widgets

credits = [['CoRot n2 eXplorer/eXporter - (c) Michael Gruberbauer, 2008'], $
           ['Version 1.1, 17/1/2008                                    '], $
	   ['Version 1.2, 21/2/2008, 1.2a, 28/2/2008                   '], $
           ['----------------------------------------------------------'], $
	   ['This program is free to use and can be destributed and    '], $
	   ['shared, as long as its distribution does not violate the  '], $ 
	   ['CoRot data access policy. Please contact the author if you'], $
	   ['find any bugs, have suggestions, questions or other       '], $
	   ['concerns dealing with this program.                       '], $
	   ['                                                          '], $
	   ['                  Email : gruberbauer@astro.univie.ac.at  ']]

credits_base = WIDGET_BASE(group_leader=base, title='About n2XX ...', /floating, row=2)
credits_text = WIDGET_TEXT(credits_base, value=credits, ysize=20)
credits_close = WIDGET_BUTTON(credits_base, value='Close', uvalue='credits_end')

WIDGET_CONTROL,credits_base, /realize
XMANAGER, 'credits', credits_base, /no_block


end

;-----------------------------------------------------------------------------
pro credits_event, ev
;event handler for the 'about'-widget

WIDGET_CONTROL,ev.id,get_uvalue=uval

case uval of 'credits_end' : begin
                          ;destroy the widget
	                  WIDGET_CONTROL, ev.top,/destroy
		        end	     
endcase

end

;-----------------------------------------------------------------------------

pro create_headerview
;produce a small widget that contains the headers in a text window

COMMON widgets
COMMON data

text = n2dat[star_ind]->getHeaders()

headerview_base = WIDGET_BASE(group_leader=base, title='Showing header(s)...', /floating, row=2)
headerview = WIDGET_TEXT(headerview_base, value=text, ysize=20, /scroll)
headerview = WIDGET_BUTTON(headerview_base, value='Close', uvalue='headerview_end')

WIDGET_CONTROL,headerview_base, /realize
XMANAGER, 'headerview', headerview_base, /no_block


end

;-----------------------------------------------------------------------------
pro headerview_event, ev
;event handler for the 'headerview'-widget

WIDGET_CONTROL,ev.id,get_uvalue=uval

case uval of 'headerview_end' : begin
                          ;destroy the widget
	                  WIDGET_CONTROL, ev.top,/destroy
		        end	     
endcase

end


;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function calc_dft, x, y

COMMON dft_widgets

n_x = n_elements(x) 
oversampling = osratio
y  = y - mean(y, /double)

dT = x[n_x-1] - x[0]
testdt = min(x[1:n_x-1] - x[0:n_x-2])
if n_x ne n_elements(y) then testdT = -1

if testdT gt 0 then begin
   ;create_abort_dft
   abort_base = WIDGET_BASE(group_leader=dft_base ,xsize=300, ysize=100, xoffset=250,yoffset=100,/floating)
   cancel = WIDGET_BUTTON(abort_base, value='Abort calculation', uvalue='abort_dft',xsize=280, ysize=80, xoffset=10, yoffset=10) 
   WIDGET_CONTROL,abort_base,/realize
   df = 1.0D/(dT*oversampling)
   n_freqs = ceil(double((ulim - llim)/df))
   amp = dblarr(n_freqs)
   arg = dblarr(n_x)
   freq = dblarr(n_freqs)
   amp_r = dblarr(n_freqs)
   ;r_sum = dblarr(n_freqs)
   ;i_sum = dblarr(n_freqs)
   amp_i = dblarr(n_freqs)
   twopi = 2.0D*!DPI
   freq = llim + dindgen(n_freqs)*df
   arg = twopi*x

   percent = 0
  
   curcossin = [[y*cos(llim*arg)], [y*sin(llim*arg)]]
 
   dcossin = [[cos(arg*df)],[sin(arg*df)]]
 
   dsintrans = reform(dcossin[*,1])/reform(dcossin[*,0])
   dcossin[*,1] = dcossin[*,0]
 
   padding_string =  '                         '

   for j=0.,n_freqs-1 do begin
      a = widget_event(cancel, /nowait)
      if a.id ne 0 then begin         
	 cancel_calc = 1
	 break
      endif
      if floor(j*100.0/n_freqs) gt percent then begin
         percent += 1
         strput, padding_string,'|', floor(percent/4.0)
         ;WIDGET_CONTROL, base, hourglass=0
         WIDGET_CONTROL, status_dft,set_value='Calculating ... ' + padding_string + strtrim(string(percent,FORMAT='(I)'),2) + ' % done'      
         ;WIDGET_CONTROL,  base, hourglass=1   
      endif	
      amp_r[j] = total(curcossin[*,0] )
      amp_i[j] = total(curcossin[*,1] )
   
      dbuf =  curcossin[*,1]   ;   curcos*dcos - cursin*dsin
      curcossin[*,1] +=  curcossin[*,0]*dsintrans  ;   cursin*dcos + curcos*dsin
      curcossin[*,0] -= dbuf*dsintrans 
   
      curcossin *=  dcossin   
   endfor

   amp = 2.0D*sqrt(amp_r*amp_r + amp_i*amp_i)/n_x 
      
   spec_arr = dblarr(n_freqs,2)
   spec_arr[*,0] = freq
   spec_arr[*,1] = amp       
   if cancel_calc eq 0 then begin
      WIDGET_CONTROL, status_dft, set_value='Done ... Click  ''Calculate !''  to calculate DFT'   
      WIDGET_CONTROL,abort_base,/destroy
      return, spec_arr      
   endif else begin
      WIDGET_CONTROL, status_dft, set_value='Aborted ... Click  ''Calculate !''  to calculate DFT'
      WIDGET_CONTROL,abort_base,/destroy 
      return, [[0.0,0.0],[0.0,0.0]]
   endelse
endif else begin
   if n_x ne n_elements(y) then ask = dialog_message(['X and Y have different numbers of data points', '- please select equally sampled data for DFT calculation. '], /information) $
   else ask = dialog_message(['The data set does not seem to be ', 'suitable for a straightforward DFT.'], /information)
   return,[[0.0, 0.0],[0.0,0.0]]
endelse


       
end

;-----------------------------------------------------------------------------

pro create_dft
;creates the DFT widget

COMMON data
COMMON dft_widgets
COMMON widgets
COMMON gui

abort_base = 0L
selected = [0,0]
dft_selection = intarr(2)
llim = 0D
ulim = 10D
llim_c = 0D
ulim_c = 10D
spec = [[0.0,0.0],[0.0,0.0]]
calced = 0
osratio = 2.0D
cancel_calc = 0
xtit = 'Frequency'
ytit = 'Amplitude'

export_strings = n2Dat[star_ind]->getExportStrings()

WIDGET_CONTROL,star_sel,sensitive=0
WIDGET_CONTROL,dft_button, sensitive=0
xrang_dft = dblarr(2)
yrang_dft = dblarr(2)
prev_zoom_dft = dblarr(4)

dft_base = WIDGET_BASE(group_leader=base, title='Calculate DFT ...', /floating,  row=3,xsize=800, xoffset=112, yoffset=0)
dft_base1 = WIDGET_BASE(dft_base, column=5)

dft_subbase1 = WIDGET_BASE(dft_base1,row = 2,frame=1)
dft_subbase2 = WIDGET_BASE(dft_base1,row = 2, frame=1)
dft_subbase3 = WIDGET_BASE(dft_base1,row = 2, frame=1)
dft_subbase4 = WIDGET_BASE(dft_base1,row = 2, frame=1)
dft_subbase5 = WIDGET_BASE(dft_base1,row = 2, frame=1)
column1 = WIDGET_LABEL(dft_subbase1, value='X')
dft_x = WIDGET_DROPLIST(dft_subbase1, value=export_strings, uvalue='dft_x')
column2 = WIDGET_LABEL(dft_subbase2, value='Y')
dft_y = WIDGET_DROPLIST(dft_subbase2, value=export_strings, uvalue='dft_y')
column3 = WIDGET_LABEL(dft_subbase3, value='Lower lim.(f):')
llim_box = WIDGET_TEXT(dft_subbase3, value='0.0000', uvalue='llim', xsize=10,/editable)
column4 = WIDGET_LABEL(dft_subbase4, value='Upper lim.(f): ')
ulim_box = WIDGET_TEXT(dft_subbase4, value='10.0000', uvalue='ulim', xsize=10, /editable)
column4 = WIDGET_LABEL(dft_subbase5, value='Oversampling:')
os_box = WIDGET_TEXT(dft_subbase5,value=' 2.00', uvalue='os', xsize=6,/editable)


dft_draw = WIDGET_DRAW(dft_base,xsize=800,ysize=500,uvalue='dft_draw',/BUTTON_EVENTS, /MOTION_EVENTS)

dft_base2 = WIDGET_BASE(dft_base, column=8)
finish = WIDGET_BUTTON(dft_base2,value='Calculate !', uvalue='ok', sensitive=0)
unzoom_dft = WIDGET_BUTTON(dft_base2,value='Show all', uvalue='unzoom_dft', sensitive=0)
status_dft = WIDGET_TEXT(dft_base2,value='Please select data for which to calculate the DFT  ...',xsize=50)
x_label = WIDGET_LABEL(dft_base2,value='X :')
x_info = WIDGET_TEXT(dft_base2, value='0.000000',xsize=13)
y_label = WIDGET_LABEL(dft_base2,value='Y :')
y_info = WIDGET_TEXT(dft_base2, value='0.000000',xsize=13)

close = WIDGET_BUTTON(dft_base2,value='Close', uvalue='close_dft')


WIDGET_CONTROL,dft_base, /realize
XMANAGER, 'dft', dft_base, /no_block, cleanup='cleanup_dft'
WIDGET_CONTROL,dft_draw,get_value=dft_drawID

end

;-----------------------------------------------------------------------------

pro cleanup_dft, ev

COMMON widgets

WIDGET_CONTROL,star_sel,sensitive=1
WIDGET_CONTROL,dft_button,sensitive=1

end

;-----------------------------------------------------------------------------

pro dft_event, ev
;event handler for the export widget

COMMON dft_widgets
COMMON widgets
COMMON data
COMMON gui


WIDGET_CONTROL,ev.id,get_uvalue=uval

case uval of 'ok' : begin
                        user_test = where(selected eq 0)
			llim_c = llim
			ulim_c = ulim
			;WIDGET_CONTROL,cancel,set_value='Abort',set_uvalue='abort_dft'
	                if user_test[0] eq -1 then begin
  			   ;WIDGET_CONTROL, dft_base, hourglass=1	
			   wset,dft_drawID
			   if bits eq 0 then begin
			      x = n2dat[star_ind]->getData(dft_selection[0])
			      y = n2dat[star_ind]->getData(dft_selection[1])
			   endif else begin
			      x = n2dat[star_ind]->getData(dft_selection[0], /clip)
			      y = n2dat[star_ind]->getData(dft_selection[1], /clip)
			   endelse   
			   spec = calc_dft(x, y)
			   loadct,0, /silent
			   if max(reform(spec[*,1])) lt 0.01 then begin
			      spec[*,1] *= 1000000D
			      ytit = 'Amplitude (ppm)'
			   endif
			   yrang_dft = [1.1D*min(spec[*,1]),0.98D*max(spec[*,1])]
			   plot,reform(spec[*,0]),reform(spec[*,1]), xrange=xrang_dft, yrange=yrang_dft, background=255, color=0, xtitle=xtit, ytitle=ytit
			   ;WIDGET_CONTROL, dft_base, hourglass=0
			   calced = 1
		        endif			   	   	
			;WIDGET_CONTROL,cancel,set_value='Close' , set_uvalue='close_dft'
			cancel_calc=0  
                     end		     
	     'close_dft' : begin
	                    ;destroy the widget if user wants to cancel
	                    WIDGET_CONTROL, dft_base,/destroy
			    wset,drawID
		          end
	     ;'abort_dft' : begin
	     ;                cancel_calc = 1
	;		     print,'Abort !!!'
	;		   end
	     	
	     'dft_x' : begin
	                  if check_type(ev.index-1) then begin
			     dft_selection[0] = ev.index-1	
			     if ev.index ne 0 then selected[0] = 1 else selected[0] = 0			  		  
			  endif else begin
			     WIDGET_CONTROL,ev.id,set_droplist_select=0
			     selected[0] = 0
		          endelse   
			  if n_elements(where(selected eq 1)) eq 2 then begin
			     WIDGET_CONTROL, finish, sensitive=1
			     WIDGET_CONTROL, status_dft, set_value='Click ''Calculate !'' to calculate DFT'
		          endif else begin
			     WIDGET_CONTROL, finish, sensitive=0
			     WIDGET_CONTROL, status_dft, set_value='Please select data for which to calculate the DFT'
			  endelse   
	              end
 	     'dft_y' : begin
	     	    	  if check_type(ev.index-1) then begin
			     dft_selection[1] = ev.index-1	
			     if ev.index ne 0 then selected[1] = 1 else selected[1] = 0			  		  
			  endif else begin
			     WIDGET_CONTROL,ev.id,set_droplist_select=0
			     selected[1] = 0
		          endelse
			  if n_elements(where(selected eq 1)) eq 2 then begin
			     WIDGET_CONTROL, finish, sensitive=1
			     WIDGET_CONTROL, status_dft, set_value='Click ''Calculate !'' to calculate DFT'
		          endif else begin
			     WIDGET_CONTROL, finish, sensitive=0
			     WIDGET_CONTROL, status_dft, set_value='Please select data for which to calculate the DFT  ...'
			  endelse
	              end 
	      'os' : begin
	                   WIDGET_CONTROL,ev.id,get_value=temp
			   osratio = double(temp[0])			   
	               end	      	
	      'llim' : begin
	                   WIDGET_CONTROL,ev.id,get_value=temp
			   xrang_dft[0] = double(temp[0])
			   llim = xrang_dft[0]
			   if calced eq 1 then begin
 			      wset,dft_drawID
			      loadct,0, /silent
			      plot,reform(spec[*,0]),reform(spec[*,1]), xrange=xrang_dft, yrange=yrang_dft, xstyle=1, background=255, color=0, xtitle=xtit, ytitle=ytit
			   endif 
	               end
	      'ulim' : begin
	                   WIDGET_CONTROL,ev.id,get_value=temp
			   xrang_dft[1] = double(temp[0])   
			   ulim = xrang_dft[1]
			   if calced eq 1 then begin
 			      wset,dft_drawID
			      loadct,0, /silent
			      plot,reform(spec[*,0]),reform(spec[*,1]), xrange=xrang_dft, yrange=yrang_dft, xstyle=1, background=255, color=0, xtitle=xtit, ytitle=ytit
			   endif  
	               end	           
	      'unzoom_dft' : begin
	                      yrang_dft = [1.1D*min(spec[*,1]),0.98D*max(spec[*,1])]
	                      xrang_dft = [llim_c, ulim_c]
			      wset,dft_drawID
			      loadct,0, /silent
			      plot,reform(spec[*,0]),reform(spec[*,1]), xrange=xrang_dft, yrange=yrang_dft, xstyle=1, background=255, color=0, xtitle=xtit, ytitle=ytit
			      WIDGET_CONTROL,unzoom_dft,sensitive=0
			end   
	      
	      'dft_draw'  : begin
	              ;handle the zooming
		      case ev.modifiers of 0 : begin
		      ;no modifier key is pressed
                            case ev.type of 			      
	                       0 : begin
			              ;mouse button has been pressed - set flag, get coordinates
			              pressed = 1
	                              x_mouse[0] = ev.x
		                      y_mouse[0] = ev.y				 
				      ;copy plot to pixel map     
				      wset,dft_drawID
			              window,/free,/pixmap,xsize=800, ysize=620
			              pixID = !D.Window			              
				      device, copy=[0, 0, 800, 620, 0, 0, dft_drawID]				      
			           end
		               1 : begin
			              ;mouse button has been released - unset flag, get coordinates
			              if dragged eq 1 then begin				      
				         pressed = 0					 
		                         x_mouse[1] = ev.x
		                         y_mouse[1] = ev.y						   
				         ;clear events, copy pixel map to plotting window				
		  	                 WIDGET_CONTROL, ev.id, clear_events=1
			                 wset,dft_drawID
			                 device, copy=[0, 0, 800, 620, 0, 0, pixID]
			                 wdelete, pixID   
				         ;convert pixel coordinates to data coordinates  
			                 temp = convert_coord(x_mouse[sort(x_mouse)], y_mouse[sort(y_mouse)], /DEVICE, /TO_DATA)
				         ;set the previous axis range for un-zooming
				         prev_zoom_dft = [xrang_dft[*], yrang_dft[*]]
				         ;set the new axis range
				         xrang_dft = reform(temp[0,*])
				         yrang_dft = reform(temp[1,*])				 
				         x_mouse = [0,0]
			                 y_mouse = [0,0]
				         ;enable the 'Total view' button				 
			                 WIDGET_CONTROL, unzoom_dft, sensitive=1
				         ;update the graph
			                 wset,dft_drawID
                                         loadct,0, /silent
                                         plot,reform(spec[*,0]),reform(spec[*,1]), xrange=xrang_dft, yrange=yrang_dft, xstyle=1, background=255, color=0, xtitle=xtit, ytitle=ytit
					 dragged = 0
				      endif else pressed = 0	 				 
			           end
		               2 : begin	
			              ;mouse has been moved -> check if zoom flag is set 
			              if pressed eq 1 then begin
				         dragged = 1
				         ;copy the previous plot to the drawing window and include a rectangle showing the zoom area
  				         wset, dft_drawID
			                 loadct, 40, /silent
			                 device, copy=[0, 0, 800, 620, 0, 0, pixID]
			                 plots, [x_mouse[0], x_mouse[0], ev.x, ev.x, x_mouse[0]], $
					        [y_mouse[0], ev.y, ev.y, y_mouse[0], y_mouse[0], y_mouse[0]], /device, color=255
					 loadct,0, /silent	
		                      endif 
				      if calced eq 1 then begin
  				         xpos = strtrim(string(convert_coord(ev.x, ev.y, /DEVICE,/TO_DATA),FORMAT='(F12.5)'),2)
					 WIDGET_CONTROL,x_info,set_value=xpos[0]
					 WIDGET_CONTROL,y_info,set_value=xpos[1]
				      endif				      
				   end
			    endcase
			 end   
			 ;CTRL is pressed
			 2 : if ev.type eq 1 then begin
			        ;modifier key 'CTRL' + mouse click -> set axis to the previous axis range				
			        xrang_dft = prev_zoom_dft[0:1]
				yrang_dft = prev_zoom_dft[2:3]
				;update graph
				wset,dft_drawID
                                loadct,0, /silent
                                plot,reform(spec[*,0]),reform(spec[*,1]), xrange=xrang_dft, yrange=yrang_dft, xstyle=1, background=255, color=0, xtitle=xtit, ytitle=ytit
			     endif
			 
			 else : break    	   
		      endcase
	       end
endcase

end

;-----------------------------------------------------------------------------
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PRO send_message, str
;puts a new message in the info box and increments the position of the top line in the
;text widget

COMMON widgets

;determine the maximal number of lines in the info-box
n_lines = 9
;get the current contents of the info_text and attach the new message
WIDGET_CONTROL, info_text, get_value=info 
WIDGET_CONTROL, info_text, set_value=[info[*],str]

;change the top line of info_text if the number of lines on display is greater
;than n_lines
if n_elements(info) gt (n_lines) then WIDGET_CONTROL, info_text, set_text_top_line = n_elements(info)-n_lines

end 
